searchNodes=[{"doc":"Hike is a library that provides elevated data types, namely Option, Either, and MayFail, in the Elixir programming language. These elevated data types offer additional functionality and safety compared to the basic data types provided by Elixir. Option : The Hike.Option type represents a value that may or may not be present. It allows you to handle cases where a value might be absent without resorting to using nil or throwing exceptions. The Option type provides functions like bind, map, and match to perform operations on the encapsulated value while handling the absence of the value gracefully. Either : The Hike.Either type represents a value that can be one of two possibilities: left or right. It is commonly used to handle cases where a computation can result in either a successful outcome or a specific error. The Either type provides functions like bind_left , bind_right , map_left , map_right , and match to perform operations on the encapsulated values based on their respective sides. MayFail : The Hike.MayFail type combines the benefits of both Option and Either . It represents a value that can either succeed or fail, similar to Either, but allows for more fine-grained error handling and composition. It provides functions like bind_success, bind_failure, map_success, and map_failure to work with the success and failure cases. The primary purpose of Hike is to enhance expressiveness, safety, and predictability in code by providing these elevated data types. They enable developers to handle different scenarios and errors in a more structured and controlled manner, reducing the reliance on exceptions and mutable state. The functional programming paradigm is promoted by focusing on immutability, pure functions, and composition, which leads to more maintainable and robust code. By using Hike, developers can write code that is easier to reason about, handle potential errors explicitly, and compose operations on elevated data types in a more concise and declarative manner. It defines a struct Hike.Option with a single field value which can either be nil or any other value of type &lt;T&gt; . a struct Hike.Either that represents an &quot;either/or&quot; value. It can contain either a left value or a right value, but not both a struct Hike.MayFail that represents an &quot;either/or&quot; value. It can contain either a Failure value or a Success value, but not both. This implementation provides shorthand functions to work with Optional data, including mapping, filtering, applying and many more functions to the value inside the Optional data. purpose of this is to provide shorthand functions at Hike module itself. ## Example iex&gt; Hike . option ( 42 ) % Hike.Option { value : 42 } iex&gt; Hike . either ( 42 ) % Hike.Either { l_value : nil , r_value : 42 , is_left? : false } iex&gt; Hike . either ( { :ok , 42 } ) % Hike.Either { l_value : nil , r_value : 42 , is_left? : false } iex&gt; Hike . either ( { :error , :ERROR_MSG } ) % Hike.Either { l_value : :ERROR_MSG , r_value : nil , is_left? : true } iex&gt; Hike . mayfail ( 9 ) % Hike.MayFail { failure : nil , success : 9 , is_success? : true } iex&gt; Hike . mayfail ( { :ok , 9 } ) % Hike.MayFail { failure : nil , success : 9 , is_success? : true } iex&gt; Hike . mayfail ( { :error , &quot;ERR_MSG&quot; } ) % Hike.MayFail { failure : &quot;ERR_MSG&quot; , success : nil , is_success? : false } iex&gt; option = Hike . option ( 42 ) ...&gt; Hike . map ( option , &amp; ( &amp;1 * 2 ) ) % Hike.Option { value : 84 } iex&gt; eth = Hike . either ( 42 ) ...&gt; Hike . map_left ( eth , &amp; ( &amp;1 * 2 ) ) % Hike.Either { l_value : nil , r_value : 42 , is_left? : false } iex&gt; eth = Hike . either ( 42 ) ...&gt; Hike . map_right ( eth , &amp; ( &amp;1 * 2 ) ) % Hike.Either { l_value : nil , r_value : 84 , is_left? : false } iex&gt; may_fail = Hike . mayfail ( 9 ) ...&gt; Hike . map_success ( may_fail , &amp; ( &amp;1 * 2 ) ) % Hike.MayFail { failure : nil , success : 18 , is_success? : true } iex&gt; defmodule HikeTest do ...&gt; alias Hike ...&gt; def divide ( x , y ) , do : x / y ...&gt; def test_divide ( x , y ) do ...&gt; Hike . try ( &amp; divide / 2 , x , y ) ...&gt; end ...&gt; end ...&gt; HikeTest . test_divide ( 4 , 2 ) |&gt; Hike . map_success ( fn x -&gt; x + 1 end ) ...&gt; HikeTest . test_divide ( 4 , 0 ) |&gt; Hike . map_success ( fn x -&gt; x + 1 end ) % Hike.MayFail { failure : nil , success : 3.0 , is_success? : true } % Hike.MayFail { failure : &quot;bad argument in arithmetic expression&quot; , success : nil , is_success? : false } Application didn't crash but successfully return error. like this example all other function can be used from Hike module itself.","ref":"Hike.html","title":"Hike","type":"module"},{"doc":"","ref":"Hike.html#apply/2","title":"Hike.apply/2","type":"function"},{"doc":"","ref":"Hike.html#apply_failure/2","title":"Hike.apply_failure/2","type":"function"},{"doc":"","ref":"Hike.html#apply_left/2","title":"Hike.apply_left/2","type":"function"},{"doc":"","ref":"Hike.html#apply_right/2","title":"Hike.apply_right/2","type":"function"},{"doc":"","ref":"Hike.html#apply_success/2","title":"Hike.apply_success/2","type":"function"},{"doc":"","ref":"Hike.html#bind/2","title":"Hike.bind/2","type":"function"},{"doc":"","ref":"Hike.html#bind_failure/2","title":"Hike.bind_failure/2","type":"function"},{"doc":"","ref":"Hike.html#bind_left/2","title":"Hike.bind_left/2","type":"function"},{"doc":"","ref":"Hike.html#bind_right/2","title":"Hike.bind_right/2","type":"function"},{"doc":"","ref":"Hike.html#bind_success/2","title":"Hike.bind_success/2","type":"function"},{"doc":"","ref":"Hike.html#either/1","title":"Hike.either/1","type":"function"},{"doc":"","ref":"Hike.html#failure/1","title":"Hike.failure/1","type":"function"},{"doc":"","ref":"Hike.html#filter/2","title":"Hike.filter/2","type":"function"},{"doc":"","ref":"Hike.html#left/1","title":"Hike.left/1","type":"function"},{"doc":"","ref":"Hike.html#map/2","title":"Hike.map/2","type":"function"},{"doc":"","ref":"Hike.html#map_failure/2","title":"Hike.map_failure/2","type":"function"},{"doc":"","ref":"Hike.html#map_left/2","title":"Hike.map_left/2","type":"function"},{"doc":"","ref":"Hike.html#map_right/2","title":"Hike.map_right/2","type":"function"},{"doc":"","ref":"Hike.html#map_success/2","title":"Hike.map_success/2","type":"function"},{"doc":"","ref":"Hike.html#match/3","title":"Hike.match/3","type":"function"},{"doc":"","ref":"Hike.html#mayfail/1","title":"Hike.mayfail/1","type":"function"},{"doc":"","ref":"Hike.html#option/0","title":"Hike.option/0","type":"function"},{"doc":"","ref":"Hike.html#option/1","title":"Hike.option/1","type":"function"},{"doc":"","ref":"Hike.html#right/1","title":"Hike.right/1","type":"function"},{"doc":"","ref":"Hike.html#success/1","title":"Hike.success/1","type":"function"},{"doc":"wraps a function call if function runs successfully will return MayFail in Success state otherwise return MayFail in Failure state.","ref":"Hike.html#try/1","title":"Hike.try/1","type":"function"},{"doc":"wraps a function with arity 1 call if function runs successfully will return MayFail in Success state otherwise return MayFail in Failure state. Example iex&gt; add1 = fn ( x ) -&gt; x + 1 end iex&gt; Hike . try ( add1 , 5 ) |&gt; Hike.MayFail . map_success ( fn x -&gt; x end ) % Hike.MayFail { failure : nil , success : 6 , is_success? : true }","ref":"Hike.html#try/2","title":"Hike.try/2","type":"function"},{"doc":"wraps a function with arity 2 call if function runs successfully will return MayFail in Success state otherwise return MayFail in Failure state. Example iex&gt; divide = fn ( x , y ) -&gt; x / y end iex&gt; Hike . try ( divide , 5 , 0 ) |&gt; ...&gt; Hike.MayFail . map_success ( fn x -&gt; { :ok , x + 1 } end ) |&gt; ...&gt; Hike.MayFail . map_failure ( fn x -&gt; String . upcase ( x ) end ) % Hike.MayFail { failure : &quot;BAD ARGUMENT IN ARITHMETIC EXPRESSION&quot; , success : nil , is_success? : false }","ref":"Hike.html#try/3","title":"Hike.try/3","type":"function"},{"doc":"wraps a function with arity 3 call if function runs successfully will return MayFail in Success state otherwise return MayFail in Failure state.","ref":"Hike.html#try/4","title":"Hike.try/4","type":"function"},{"doc":"wraps a function with arity 4 call if function runs successfully will return MayFail in Success state otherwise return MayFail in Failure state.","ref":"Hike.html#try/5","title":"Hike.try/5","type":"function"},{"doc":"","ref":"Hike.html#t:exception/0","title":"Hike.exception/0","type":"type"},{"doc":"generic input type &lt;T&gt; .","ref":"Hike.html#t:t/0","title":"Hike.t/0","type":"type"},{"doc":"generic input type &lt;TArg1&gt; .","ref":"Hike.html#t:tArg1/0","title":"Hike.tArg1/0","type":"type"},{"doc":"generic input type &lt;TArg2&gt; .","ref":"Hike.html#t:tArg2/0","title":"Hike.tArg2/0","type":"type"},{"doc":"generic input type &lt;TArg3&gt; .","ref":"Hike.html#t:tArg3/0","title":"Hike.tArg3/0","type":"type"},{"doc":"generic input type &lt;TArg4&gt; .","ref":"Hike.html#t:tArg4/0","title":"Hike.tArg4/0","type":"type"},{"doc":"generic return type &lt;TR&gt; .","ref":"Hike.html#t:tr/0","title":"Hike.tr/0","type":"type"},{"doc":"Hike.Either represents a value that can be one of two possibilities: either a Left state or a Right state. It is commonly used in error handling or when a function can return different types of results. Creating an Either To create an Either instance, you can use the Hike.Either.right/1 and Hike.Either.left/1 function to wrap a value in right and left state respectively : iex&gt; Hike.Either . right ( 5 ) % Hike.Either { l_value : nil , r_value : 5 , is_left? : false } iex&gt; Hike.Either . left ( &quot;Hello&quot; ) % Hike.Either { l_value : &quot;Hello&quot; , r_value : nil , is_left? : true } # Example can be written with `Either` # Define a User struct defmodule User do @derive Jason.Encoder defstruct [ :id , :age , :name ] end defmodule TestHike do # Import the Hike.Either module import Hike.Either # Simulating a database fetch function @spec fetch_user ( number ) :: Hike.Either . either ( % User { } ) | Hike.Either . either ( ) def fetch_user ( id ) do # Simulating a database query to fetch a user by ID # Returns an Either&lt;User, string&gt; with left(user) if the user is found # Returns an Either&lt;User, string&gt; with right(&quot;User not found&quot;) if the user is not found case id do 1 -&gt; left ( % User { id : 1 , age : 30 , name : &quot;Vineet Sharma&quot; } ) 2 -&gt; left ( % User { id : 2 , age : 20 , name : &quot;Jane Smith&quot; } ) _ -&gt; right ( &quot;User not found&quot; ) end end # Function to update the user&#39;s name to uppercase if possible # This function takes a User struct and returns an Either&lt;User, string&gt; def update_name_to_uppercase ( user ) do case user . name do nil -&gt; right ( &quot;User name is missing&quot; ) name -&gt; left ( % User { user | name : String . upcase ( name ) } ) end end # Function to increase the user&#39;s age by one # This function takes a User struct and returns a real data type User with updated values. def increase_age_by_1 ( user ) do % User { user | age : user . age + 1 } end # Function to print a user struct as a JSON-represented string def print_user_as_json ( user ) do user |&gt; Jason . encode! ( ) |&gt; IO . puts ( ) end # Example: Fetching a user from the database, updating the name, and matching the result def test_user ( ) do user_id = 1 # Fetch the user from the database fetch_user ( user_id ) # Update the name to uppercase using bind |&gt; bind_left ( &amp; update_name_to_uppercase / 1 ) # Increase the age by one using map |&gt; map_left ( &amp; increase_age_by_1 / 1 ) # Print the user as a JSON string using map |&gt; map_left ( &amp; print_user_as_json / 1 ) # finally match the respective result with a appropriate function. |&gt; match ( fn x -&gt; x end , fn err -&gt; err end ) user_id = 3 # Fetch the user from the database fetch_user ( user_id ) # Update the name to uppercase using bind |&gt; bind_left ( &amp; update_name_to_uppercase / 1 ) # Increase the age by one using map |&gt; map_left ( &amp; increase_age_by_1 / 1 ) # Print the user as a JSON string using map |&gt; map_left ( &amp; print_user_as_json / 1 ) # finally match the respective result with a appropriate function. |&gt; match ( fn x -&gt; x end , fn err -&gt; err end ) end end #output iex&gt; TestHike.test_user #user_id = 1 {&quot;age&quot;:31,&quot;id&quot;:1,&quot;name&quot;:&quot;VINEET SHARMA&quot;} #user_id = 3 &quot;User not found&quot;","ref":"Hike.Either.html","title":"Hike.Either","type":"module"},{"doc":"%Hike.Either{l_value: t_left(), r_value: t_right(), is_left?:boolean()} is a struct that represents an &quot;either/or&quot; value. It can contain either a left value or a right value, but not both. l_value : the left value (if is_left? is true) r_value : the right value (if is_left? is false) is_left? : a boolean flag indicating whether the value is a left value ( true ) or a right value ( false )","ref":"Hike.Either.html#__struct__/0","title":"Hike.Either.__struct__/0","type":"function"},{"doc":"apply_left applies a given function to a given Either if Either is in left state and return a new Either with new transformed value in Left state else return new Either in Right state with existing right value. Examples iex&gt; ( Either . left 5 ) |&gt; Either . apply_left ( fn num -&gt; num + num end ) % Hike.Either { l_value : 10 , r_value : nil , is_left? : true } iex&gt; ( Either . right 5 ) |&gt; Either . apply_left ( fn num -&gt; num + num end ) % Hike.Either { l_value : nil , r_value : 5 , is_left? : false }","ref":"Hike.Either.html#apply_left/2","title":"Hike.Either.apply_left/2","type":"function"},{"doc":"Applies the provided function func to the left value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Either . left ( &quot;Hello&quot; ) end ) ...&gt; applied_task = apply_left_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( 10 ) end ) ...&gt; applied_task = apply_left_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : nil , r_value : &quot;Hello&quot; , is_left? : false }","ref":"Hike.Either.html#apply_left_async/2","title":"Hike.Either.apply_left_async/2","type":"function"},{"doc":"apply_right applies a given function to a given Either if Either is in right state and return a new Either with new transformed value in Right state else return new Either in Left state with existing left value. Examples iex&gt; ( Either . right &quot;hello&quot; ) |&gt; Either . apply_right ( fn str -&gt; String . upcase ( str ) end ) % Hike.Either { l_value : nil , r_value : &quot;HELLO&quot; , is_left? : false } iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . apply_right ( fn str -&gt; String . upcase ( str ) end ) % Hike.Either { l_value : &quot;hello&quot; , r_value : nil , is_left? : true }","ref":"Hike.Either.html#apply_right/2","title":"Hike.Either.apply_right/2","type":"function"},{"doc":"Applies the provided function func to the right value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; % Hike.Either { right : &quot;Hello&quot; } end ) ...&gt; applied_task = apply_right_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { left : nil , right : &quot;HELLO&quot; , is_left? : false } iex&gt; task = Task . async ( fn -&gt; % Hike.Either { left : 10 } end ) ...&gt; applied_task = apply_right_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { left : 10 , right : nil , is_left? : true }","ref":"Hike.Either.html#apply_right_async/2","title":"Hike.Either.apply_right_async/2","type":"function"},{"doc":"Binds a function that returns an Either value for an Either in the left state. If the input Either is in the right state, the function is not executed and the input Either is returned as is. Examples iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . bind_left ( fn str -&gt; Either . left ( String . upcase ( str ) ) end ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . bind_left ( fn str -&gt; Either . right ( String . upcase ( str ) ) end ) % Hike.Either { l_value : nil , r_value : &quot;HELLO&quot; , is_left? : false } iex&gt; ( Either . right &quot;hello&quot; ) |&gt; Either . bind_left ( fn str -&gt; Either . left ( String . upcase ( str ) ) end ) % Hike.Either { l_value : nil , r_value : &quot;hello&quot; , is_left? : false }","ref":"Hike.Either.html#bind_left/2","title":"Hike.Either.bind_left/2","type":"function"},{"doc":"Applies the provided function binder to the left value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Either . left ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_left_async ( task , fn str -&gt; Either . left ( String . upcase ( str ) ) end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_left_async ( task , fn str -&gt; Either . left ( String . upcase ( str ) ) end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : nil , r_value : &quot;Hello&quot; , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( 10 ) end ) ...&gt; applied_task = bind_left_async ( task , fn num -&gt; num * num end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : nil , r_value : 10 , is_left? : false }","ref":"Hike.Either.html#bind_left_async/2","title":"Hike.Either.bind_left_async/2","type":"function"},{"doc":"Binds a function that returns an Either value to an Either in the right state. If the input Either is in the left state, the function is not executed and the input Either is returned as is. Examples iex&gt; ( Either . right &quot;hello&quot; ) |&gt; Either . bind_right ( fn str -&gt; Either . right ( String . upcase ( str ) ) end ) % Hike.Either { l_value : nil , r_value : &quot;HELLO&quot; , is_left? : false } iex&gt; ( Either . right &quot;hello&quot; ) |&gt; Either . bind_right ( fn str -&gt; Either . left ( String . upcase ( str ) ) end ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . bind_right ( fn str -&gt; Either . right ( String . upcase ( str ) ) end ) % Hike.Either { l_value : &quot;hello&quot; , r_value : nil , is_left? : true }","ref":"Hike.Either.html#bind_right/2","title":"Hike.Either.bind_right/2","type":"function"},{"doc":"Applies the provided function binder to the right value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Either . left ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_right_async ( task , fn str -&gt; Either . left ( String . upcase ( str ) ) end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : &quot;Hello&quot; , r_value : nil , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_right_async ( task , fn str -&gt; Either . left ( String . upcase ( str ) ) end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( 10 ) end ) ...&gt; applied_task = bind_right_async ( task , fn num -&gt; num * num end ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : nil , r_value : 100 , is_left? : false }","ref":"Hike.Either.html#bind_right_async/2","title":"Hike.Either.bind_right_async/2","type":"function"},{"doc":"Create new Either from result. if result is {:ok, val} Either will be in right state. else if result is {:error, val} Either will be in left state. with respective value val in respective side.","ref":"Hike.Either.html#from_result/1","title":"Hike.Either.from_result/1","type":"function"},{"doc":"Check whether an either is in Left state or not . Examples iex&gt; right ( &quot;foo bar&quot; ) |&gt; Either . is_left? ( ) false iex&gt; left ( &quot;foo bar&quot; ) |&gt; Either . is_left? ( ) true","ref":"Hike.Either.html#is_left?/1","title":"Hike.Either.is_left?/1","type":"function"},{"doc":"","ref":"Hike.Either.html#is_not_nil/1","title":"Hike.Either.is_not_nil/1","type":"macro"},{"doc":"Check whether an either is in Right state or not . Examples iex&gt; right ( &quot;foo bar&quot; ) |&gt; Either . is_right? ( ) true iex&gt; left ( &quot;foo bar&quot; ) |&gt; Either . is_right? ( ) false","ref":"Hike.Either.html#is_right?/1","title":"Hike.Either.is_right?/1","type":"function"},{"doc":"Creates a new Left Either object with a given value for the left side. Examples iex&gt; left ( &quot;foo bar&quot; ) % Hike.Either { l_value : &quot;foo bar&quot; , r_value : nil , is_left? : true }","ref":"Hike.Either.html#left/1","title":"Hike.Either.left/1","type":"function"},{"doc":"Maps the value of the Either from left state using the given function. If the Either is in the right state, the function returns the Either unchanged. If the Either is in the left state, the function applies the given function to the value of the left state, and returns a new Either with the transformed value. Examples iex&gt; ( Either . left 5 ) |&gt; Either . map_left ( fn num -&gt; num + num end ) % Hike.Either { l_value : 10 , r_value : nil , is_left? : true } iex&gt; ( Either . right 5 ) |&gt; Either . map_left ( fn num -&gt; num + num end ) % Hike.Either { l_value : nil , r_value : 5 , is_left? : false } iex&gt; either = % Either { l_value : &quot;hello&quot; , is_left? : true } ...&gt; new_either = Either . map_left ( either , &amp; String . upcase / 1 ) % Hike.Either { l_value : &quot;HELLO&quot; , is_left? : true } iex&gt; either = % Either { r_value : 10 , is_left? : false } ...&gt; new_either = Either . map_left ( either , &amp; String . downcase / 1 ) % Hike.Either { r_value : 10 , is_left? : false }","ref":"Hike.Either.html#map_left/2","title":"Hike.Either.map_left/2","type":"function"},{"doc":"Applies the provided function mapper to the left value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Either . left ( &quot;Hello&quot; ) end ) ...&gt; applied_task = map_left_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : &quot;HELLO&quot; , r_value : nil , is_left? : true } iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( 10 ) end ) ...&gt; applied_task = map_left_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { l_value : nil , r_value : &quot;Hello&quot; , is_left? : false }","ref":"Hike.Either.html#map_left_async/2","title":"Hike.Either.map_left_async/2","type":"function"},{"doc":"Maps the value of the Either from right state using the given function. If the Either is in the left state, the function returns the Either unchanged. If the Either is in the right state, the function applies the given function to the value of the right state, and returns a new Either with the transformed value. Examples iex&gt; ( Either . right &quot;hello&quot; ) |&gt; Either . map_right ( fn str -&gt; String . upcase ( str ) end ) % Hike.Either { l_value : nil , r_value : &quot;HELLO&quot; , is_left? : false } iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . map_right ( fn str -&gt; String . upcase ( str ) end ) % Hike.Either { l_value : &quot;hello&quot; , r_value : nil , is_left? : true }","ref":"Hike.Either.html#map_right/2","title":"Hike.Either.map_right/2","type":"function"},{"doc":"Applies the provided function mapper to the right value of Either wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; % Hike.Either { right : &quot;Hello&quot; } end ) ...&gt; applied_task = map_right_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { left : nil , right : &quot;HELLO&quot; , is_left? : false } iex&gt; task = Task . async ( fn -&gt; % Hike.Either { left : 10 } end ) ...&gt; applied_task = map_right_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.Either { left : 10 , right : nil , is_left? : true }","ref":"Hike.Either.html#map_right_async/2","title":"Hike.Either.map_right_async/2","type":"function"},{"doc":"Matches an Either value and applies the corresponding function. Examples iex&gt; ( Either . left &quot;hello&quot; ) |&gt; Either . match ( fn str -&gt; String . upcase ( str ) end , fn ( ) -&gt; &quot;NOT FOUND&quot; end ) &quot;HELLO&quot; iex&gt; ( Either . right 4 ) |&gt; Either . match ( fn num -&gt; num * num end , fn num -&gt; num + num end ) 8","ref":"Hike.Either.html#match/3","title":"Hike.Either.match/3","type":"function"},{"doc":"Matches an Either value wrapped in a Task and applies the corresponding function asynchronously. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Either . left ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_left_async ( task , fn str -&gt; Either . left ( String . upcase ( str ) ) end ) ...&gt; match_async ( applied_task , fn x -&gt; &quot;value is : &quot; &lt;&gt; x &lt;&gt; &quot;.&quot; end , fn ( _ ) -&gt; &quot;No Result Found.&quot; end ) &quot;value is : HELLO.&quot; iex&gt; task = Task . async ( fn -&gt; Hike.Either . right ( &quot;Hello&quot; ) end ) ...&gt; applied_task = map_left_async ( task , &amp; String . upcase / 1 ) ...&gt; match_async ( applied_task , fn x -&gt; x end , fn ( y ) -&gt; &quot;value is : &quot; &lt;&gt; y &lt;&gt; &quot;.&quot; end ) &quot;value is : Hello.&quot;","ref":"Hike.Either.html#match_async/3","title":"Hike.Either.match_async/3","type":"function"},{"doc":"Creates a new Either in Right state with a given value. Examples iex&gt; right ( &quot;foo bar&quot; ) % Hike.Either { l_value : nil , r_value : &quot;foo bar&quot; , is_left? : false }","ref":"Hike.Either.html#right/1","title":"Hike.Either.right/1","type":"function"},{"doc":"convert Either form Right state to MayFail Success state and Either from Left state to MayFail Failure state. Example iex&gt; Hike . left ( &quot;:error&quot; ) |&gt; Hike.Either . to_mayfail ( ) % Hike.MayFail { failure : &quot;:error&quot; , success : nil , is_success? : false } iex&gt; Hike . right ( 9 ) |&gt; Hike.Either . to_mayfail % Hike.MayFail { failure : nil , success : 9 , is_success? : true }","ref":"Hike.Either.html#to_mayfail/1","title":"Hike.Either.to_mayfail/1","type":"function"},{"doc":"convert Either from Right state to Option Some state and Either from Left state to Option None state. Example iex&gt; Hike . right ( 9 ) |&gt; Hike.Either . to_option ( ) % Hike.Option { value : 9 } iex&gt; Hike . left ( &quot;:error&quot; ) |&gt; Hike.Either . to_option ( ) % Hike.Option { value : nil }","ref":"Hike.Either.html#to_option/1","title":"Hike.Either.to_option/1","type":"function"},{"doc":"binder() represent a binding(mapping) function which take no parameter and return an Either of type &lt;TR&gt; . Example @spec whatever ( ) :: Hike.Either . either_right ( atom ) def whatever ( ) , do : right ( :ok ) @spec whatever ( ) :: Hike.Either . either_left ( :error ) def whatever ( ) , do : left ( :error )","ref":"Hike.Either.html#t:binder/0","title":"Hike.Either.binder/0","type":"type"},{"doc":"binder(t) represent a binding(mapping) function which take a parameter of type &lt;T&gt; and return an Either of type &lt;TR&gt; . Example @spec square ( pos_number ) :: Hike.Either . either_right ( number ) def square ( x ) when x &gt; 0 , do : x * x |&gt; right ( ) @spec square ( number ) :: Hike.Either . either_left ( :error ) | Hike.Either . either_right ( number ) def square ( x ) do case x when x &lt; 0 do true -&gt; left ( { :error , &quot;negative number&quot; } ) false -&gt; square ( x ) end end","ref":"Hike.Either.html#t:binder/1","title":"Hike.Either.binder/1","type":"type"},{"doc":"represent a type of Either that could be in either Left or Right state","ref":"Hike.Either.html#t:either/0","title":"Hike.Either.either/0","type":"opaque"},{"doc":"represent a type of Either that could be in either Left or Right state with a value of given type.","ref":"Hike.Either.html#t:either/2","title":"Hike.Either.either/2","type":"type"},{"doc":"represent a type of Either in Left state.","ref":"Hike.Either.html#t:either_left/0","title":"Hike.Either.either_left/0","type":"type"},{"doc":"represent a type of Either in Left state.","ref":"Hike.Either.html#t:either_left/1","title":"Hike.Either.either_left/1","type":"type"},{"doc":"Elevated data type of Either struct that represents Right state.","ref":"Hike.Either.html#t:either_right/0","title":"Hike.Either.either_right/0","type":"type"},{"doc":"Elevated data type of Either struct that represents Right state and have a value of type &lt;T&gt; .","ref":"Hike.Either.html#t:either_right/1","title":"Hike.Either.either_right/1","type":"type"},{"doc":"func() represent a function which take no parameter and return value of type &lt;TR&gt; . Example @spec add_one ( number ) :: number def add_one ( x ) , do : x + 1","ref":"Hike.Either.html#t:func/0","title":"Hike.Either.func/0","type":"type"},{"doc":"func(t) represent a function which take a parameter of type &lt;T&gt; and return a value of type &lt;TR&gt; . Example @spec add_one ( number ) :: number def add_one ( x ) , do : x + 1","ref":"Hike.Either.html#t:func/1","title":"Hike.Either.func/1","type":"type"},{"doc":"mapper() represent a mapping function which take no parameter and return a value of type &lt;TR&gt; . Example @spec whatever ( ) :: atom def whatever ( ) , do : :ok","ref":"Hike.Either.html#t:mapper/0","title":"Hike.Either.mapper/0","type":"type"},{"doc":"mapper(t) represent a mapping function which take a parameter of type &lt;T&gt; and return a value of type &lt;TR&gt; . Example @spec square ( number ) :: number def square ( x ) , do : x * x","ref":"Hike.Either.html#t:mapper/1","title":"Hike.Either.mapper/1","type":"type"},{"doc":"generic input type &lt;T&gt; .","ref":"Hike.Either.html#t:t/0","title":"Hike.Either.t/0","type":"type"},{"doc":"generic input type &lt;T_Left&gt; represent a type of value on Left state.","ref":"Hike.Either.html#t:t_left/0","title":"Hike.Either.t_left/0","type":"type"},{"doc":"generic input type &lt;T_Right&gt; represent a type of value on Right state.","ref":"Hike.Either.html#t:t_right/0","title":"Hike.Either.t_right/0","type":"type"},{"doc":"generic return type &lt;TR&gt; .","ref":"Hike.Either.html#t:tr/0","title":"Hike.Either.tr/0","type":"type"},{"doc":"Hike.MayFail represents a value that may either succeed with a value or fail with an error. It combines the functionality of Hike.Option and Hike.Either , making it suitable for scenarios where a value can be optional and can also potentially fail. Creating a MayFail To create a MayFail instance, you can use the Hike.MayFail.success/1 function to wrap a success value: iex&gt; may_fail = Hike.MayFail . success ( 42 ) % Hike.MayFail { failure : nil , success : 42 , is_success? : true } iex&gt; may_fail = Hike.MayFail . failure ( :error ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } # same example can be rewritten with `MayFail` # Define a User struct defmodule User do @derive Jason.Encoder defstruct [ :id , :age , :name ] end defmodule TestHike do # Import the Hike.MayFail module import Hike.MayFail # Simulating a database fetch function @spec fetch_user ( number ) :: Hike.MayFail . mayfail ( String . t ( ) ) | Hike.MayFail . mayfail_success ( % User { } ) def fetch_user ( id ) do # Simulating a database query to fetch a user by ID # Returns an MayFail&lt;string, User&gt; with success(user) if the user is found # Returns an MayFail&lt;string, User&gt; with failure(&quot;User not found&quot;) if the user is not found case id do 1 -&gt; success ( % User { id : 1 , age : 30 , name : &quot;Vineet Sharma&quot; } ) 2 -&gt; success ( % User { id : 2 , age : 20 , name : nil } ) _ -&gt; failure ( &quot;User not found&quot; ) end end # Function to update the user&#39;s name to uppercase if possible # This function takes a User struct and returns an MayFail&lt;string, User&gt; def update_name_to_uppercase ( user ) do case user . name do nil -&gt; failure ( &quot;User name is missing&quot; ) name -&gt; success ( % User { user | name : String . upcase ( name ) } ) end end # Function to increase the user&#39;s age by one # This function takes a User struct and returns a real data type User with updated values. def increase_age_by_1 ( user ) do % User { user | age : user . age + 1 } end # Function to print a user struct as a JSON-represented string def print_user_as_json ( user ) do user |&gt; Jason . encode! ( ) |&gt; IO . puts ( ) end @spec test_user ( ) :: :ok def test_user ( ) do fetch_user ( 1 ) |&gt; bind_success ( &amp; update_name_to_uppercase / 1 ) |&gt; map_success ( &amp; increase_age_by_1 / 1 ) |&gt; IO . inspect ( ) |&gt; match ( &amp; IO . puts / 1 , &amp; print_user_as_json / 1 ) fetch_user ( 2 ) |&gt; bind_success ( &amp; update_name_to_uppercase / 1 ) |&gt; map_success ( &amp; increase_age_by_1 / 1 ) |&gt; IO . inspect ( ) |&gt; match ( &amp; IO . puts / 1 , &amp; print_user_as_json / 1 ) fetch_user ( 3 ) |&gt; bind_success ( &amp; update_name_to_uppercase / 1 ) |&gt; map_success ( &amp; increase_age_by_1 / 1 ) |&gt; IO . inspect ( ) |&gt; match ( &amp; IO . puts / 1 , &amp; print_user_as_json / 1 ) :ok end end iex&gt; TestHike . test_user # user id =1 % Hike.MayFail { failure : nil , success : % User { id : 1 , age : 31 , name : &quot;VINEET SHARMA&quot; } , is_success? : true } { &quot;age&quot; : 31 , &quot;id&quot; : 1 , &quot;name&quot; :&quot;VINEET SHARMA&quot; } # user id = 2 % Hike.MayFail { failure : &quot;User name is missing&quot; , success : nil , is_success? : false } User name is missing #user id = 3 % Hike.MayFail { failure : &quot;User not found&quot; , success : nil , is_success? : false } User not found :ok","ref":"Hike.MayFail.html","title":"Hike.MayFail","type":"module"},{"doc":"%Hike.Mayfail{failure: t_failure(), success: t_success(), is_success?:boolean()} is a struct that represents an &quot;either/or&quot; value. It can contain either a Failure value or a Success value, but not both. failure : the failure value (if is_success? is false) success : the success value (if is_success? is true) is_success? : a boolean flag indicating whether the value is a success value ( true ) or a failure value ( false )","ref":"Hike.MayFail.html#__struct__/0","title":"Hike.MayFail.__struct__/0","type":"function"},{"doc":"Example iex&gt; apply_func = fn x -&gt; x + 2 end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . apply_failure ( apply_func ) % Hike.MayFail { failure : 3 , success : nil , is_success? : false } iex&gt; apply_func = fn x -&gt; x + 2 end ...&gt; MayFail . success ( 1 ) |&gt; MayFail . apply_failure ( apply_func ) % Hike.MayFail { failure : nil , success : 1 , is_success? : true }","ref":"Hike.MayFail.html#apply_failure/2","title":"Hike.MayFail.apply_failure/2","type":"function"},{"doc":"Applies the provided function func to the failure value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; applied_task = apply_failure_async ( task , fn x -&gt; x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; applied_task = apply_failure_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 10 , is_success? : true }","ref":"Hike.MayFail.html#apply_failure_async/2","title":"Hike.MayFail.apply_failure_async/2","type":"function"},{"doc":"Example iex&gt; apply_func = fn x -&gt; ( x + 2 ) end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . apply_success ( apply_func ) % Hike.MayFail { failure : 1 , success : nil , is_success? : false } iex&gt; MayFail . success ( 1 ) |&gt; MayFail . apply_success ( apply_func ) % Hike.MayFail { failure : nil , success : 3 , is_success? : true }","ref":"Hike.MayFail.html#apply_success/2","title":"Hike.MayFail.apply_success/2","type":"function"},{"doc":"Applies the provided function func to the Success value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; applied_task = apply_success_async ( task , fn x -&gt; x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; applied_task = apply_success_async ( task , fn x -&gt; x * x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 100 , is_success? : true }","ref":"Hike.MayFail.html#apply_success_async/2","title":"Hike.MayFail.apply_success_async/2","type":"function"},{"doc":"Binds a function that returns a MayFail value for an MayFail in the Failure state. If the input is in the Success state, the function is not executed and the input is returned as it is. Example iex&gt; binder = fn x -&gt; MayFail . success ( x + 2 ) end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . bind_failure ( binder ) % Hike.MayFail { failure : nil , success : 3 , is_success? : true } iex&gt; binder = fn x -&gt; MayFail . failure ( x + 2 ) end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . bind_failure ( binder ) % Hike.MayFail { failure : 3 , success : nil , is_success? : false } iex&gt; MayFail . success ( 1 ) |&gt; MayFail . bind_failure ( binder ) % Hike.MayFail { failure : nil , success : 1 , is_success? : true }","ref":"Hike.MayFail.html#bind_failure/2","title":"Hike.MayFail.bind_failure/2","type":"function"},{"doc":"Applies the provided function binder to the failure value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; binder = fn x -&gt; MayFail . success ( x + 2 ) end ...&gt; applied_task = bind_failure_async ( task , binder ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; applied_task = bind_failure_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 10 , is_success? : true }","ref":"Hike.MayFail.html#bind_failure_async/2","title":"Hike.MayFail.bind_failure_async/2","type":"function"},{"doc":"Binds a function that returns a MayFail value for an MayFail in the Failure state. If the input is in the Success state, the function is not executed and the input is returned as it is. Example iex&gt; binder = fn x -&gt; MayFail . success ( x + 2 ) end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . bind_success ( binder ) % Hike.MayFail { failure : 1 , success : nil , is_success? : false } iex&gt; MayFail . success ( 1 ) |&gt; MayFail . bind_success ( binder ) % Hike.MayFail { failure : nil , success : 3 , is_success? : true } iex&gt; binder = fn x -&gt; MayFail . failure ( x + 2 ) end ...&gt; MayFail . success ( 1 ) |&gt; MayFail . bind_success ( binder ) % Hike.MayFail { failure : 3 , success : nil , is_success? : false }","ref":"Hike.MayFail.html#bind_success/2","title":"Hike.MayFail.bind_success/2","type":"function"},{"doc":"Applies the provided function binder to the success value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; binder = fn x -&gt; MayFail . success ( x + 2 ) end ...&gt; applied_task = bind_success_async ( task , binder ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; binder = fn x -&gt; MayFail . success ( x + 2 ) end ...&gt; applied_task = bind_success_async ( task , binder ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 12 , is_success? : true }","ref":"Hike.MayFail.html#bind_success_async/2","title":"Hike.MayFail.bind_success_async/2","type":"function"},{"doc":"","ref":"Hike.MayFail.html#failure/1","title":"Hike.MayFail.failure/1","type":"function"},{"doc":"Check whether MayFail is in Failure state or not. Example iex&gt; Hike.MayFail . success ( 4 ) |&gt; Hike.MayFail . is_failure? false iex&gt; Hike.MayFail . failure ( &quot;fail&quot; ) |&gt; Hike.MayFail . is_failure? true","ref":"Hike.MayFail.html#is_failure?/1","title":"Hike.MayFail.is_failure?/1","type":"function"},{"doc":"Check whether MayFail is in Success state or not. Example iex&gt; Hike.MayFail . success ( 4 ) |&gt; Hike.MayFail . is_success? true iex&gt; Hike.MayFail . failure ( &quot;fail&quot; ) |&gt; Hike.MayFail . is_success? false","ref":"Hike.MayFail.html#is_success?/1","title":"Hike.MayFail.is_success?/1","type":"function"},{"doc":"Example iex&gt; mapper = fn x -&gt; x + 2 end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . map_failure ( mapper ) % Hike.MayFail { failure : 3 , success : nil , is_success? : false } iex&gt; MayFail . success ( 1 ) |&gt; MayFail . map_failure ( mapper ) % Hike.MayFail { failure : nil , success : 1 , is_success? : true }","ref":"Hike.MayFail.html#map_failure/2","title":"Hike.MayFail.map_failure/2","type":"function"},{"doc":"Applies the provided function mapper to the failure value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; applied_task = map_failure_async ( task , fn x -&gt; x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; applied_task = map_failure_async ( task , &amp; String . upcase / 1 ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 10 , is_success? : true }","ref":"Hike.MayFail.html#map_failure_async/2","title":"Hike.MayFail.map_failure_async/2","type":"function"},{"doc":"Example iex&gt; mapper = fn x -&gt; ( x + 2 ) end ...&gt; MayFail . failure ( 1 ) |&gt; MayFail . map_success ( mapper ) % Hike.MayFail { failure : 1 , success : nil , is_success? : false } iex&gt; MayFail . success ( 1 ) |&gt; MayFail . map_success ( mapper ) % Hike.MayFail { failure : nil , success : 3 , is_success? : true }","ref":"Hike.MayFail.html#map_success/2","title":"Hike.MayFail.map_success/2","type":"function"},{"doc":"Applies the provided function mapper to the success value of MayFail wrapped in a Task and returns a new task with the result. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( :error ) end ) ...&gt; applied_task = map_success_async ( task , fn x -&gt; x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( 10 ) end ) ...&gt; applied_task = map_success_async ( task , fn x -&gt; x * x end ) ...&gt; Task . await ( applied_task ) % Hike.MayFail { failure : nil , success : 100 , is_success? : true }","ref":"Hike.MayFail.html#map_success_async/2","title":"Hike.MayFail.map_success_async/2","type":"function"},{"doc":"Matches an Mayfail value and applies the corresponding function. Example iex&gt; Hike.MayFail . success ( 4 ) |&gt; Hike.MayFail . match ( fn x -&gt; x + 3 end , fn y -&gt; y + 2 end ) 6","ref":"Hike.MayFail.html#match/3","title":"Hike.MayFail.match/3","type":"function"},{"doc":"Matches an MayFail value wrapped in a Task and applies the corresponding function asynchronously. Examples iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . failure ( &quot;Hello&quot; ) end ) ...&gt; applied_task = bind_failure_async ( task , fn str -&gt; MayFail . failure ( String . upcase ( str ) ) end ) ...&gt; match_async ( applied_task , fn x -&gt; &quot;value is : &quot; &lt;&gt; x &lt;&gt; &quot;.&quot; end , fn ( _ ) -&gt; &quot;No Result Found.&quot; end ) &quot;value is : HELLO.&quot; iex&gt; task = Task . async ( fn -&gt; Hike.MayFail . success ( &quot;Hello&quot; ) end ) ...&gt; applied_task = map_success_async ( task , &amp; String . upcase / 1 ) ...&gt; match_async ( applied_task , fn x -&gt; x end , fn ( y ) -&gt; &quot;value is : &quot; &lt;&gt; y &lt;&gt; &quot;.&quot; end ) &quot;value is : HELLO.&quot;","ref":"Hike.MayFail.html#match_async/3","title":"Hike.MayFail.match_async/3","type":"function"},{"doc":"","ref":"Hike.MayFail.html#success/1","title":"Hike.MayFail.success/1","type":"function"},{"doc":"convert MayFail into respective Either . if MayFail from Success state, Either will be return in Right state. and MayFail from Failure state, Either will convert to Either Left state. Example iex&gt; success ( 9 ) |&gt; Hike.MayFail . to_either % Hike.Either { l_value : nil , r_value : 9 , is_left? : false } iex&gt; failure ( :error ) |&gt; Hike.MayFail . to_either % Hike.Either { l_value : :error , r_value : nil , is_left? : true }","ref":"Hike.MayFail.html#to_either/1","title":"Hike.MayFail.to_either/1","type":"function"},{"doc":"convert MayFail from Success state to Option Some state and MayFail from Failure state to Option None state. Example iex&gt; success ( 9 ) |&gt; Hike.MayFail . to_option ( ) % Hike.Option { value : 9 } iex&gt; failure ( &quot;:error&quot; ) |&gt; Hike.MayFail . to_option ( ) % Hike.Option { value : nil }","ref":"Hike.MayFail.html#to_option/1","title":"Hike.MayFail.to_option/1","type":"function"},{"doc":"binder() represent a binding(mapping) function which take no argument and return a Mayfail of type &lt;TR&gt; . Example iex&gt; success_bind_func = fn ( ) -&gt; Hike.MayFail . success ( :ok ) end iex&gt; failure_bind_func = fn ( ) -&gt; Hike.MayFail . failure ( :error ) end","ref":"Hike.MayFail.html#t:binder/0","title":"Hike.MayFail.binder/0","type":"type"},{"doc":"binder(t) represent a binding(mapping) function which take an argument of type &lt;T&gt; and return a Mayfail of type &lt;TR&gt; . Example iex&gt; success_bind_func = fn ( x ) -&gt; Mayfail . success ( x ) end iex&gt; failure_bind_func = fn ( y ) -&gt; Mayfail . failure ( y ) end","ref":"Hike.MayFail.html#t:binder/1","title":"Hike.MayFail.binder/1","type":"type"},{"doc":"func() represent a function which take no argument and return value of type &lt;TR&gt; .","ref":"Hike.MayFail.html#t:func/0","title":"Hike.MayFail.func/0","type":"type"},{"doc":"func(t) represent a function which take an argument of type &lt;T&gt; and return a value of type &lt;TR&gt; .","ref":"Hike.MayFail.html#t:func/1","title":"Hike.MayFail.func/1","type":"type"},{"doc":"mapper() represent a mapping function which take no argument and return a value of type &lt;TR&gt; .","ref":"Hike.MayFail.html#t:mapper/0","title":"Hike.MayFail.mapper/0","type":"type"},{"doc":"mapper(t) represent a mapping function which take an argument of type &lt;T&gt; and return a value of type &lt;TR&gt; .","ref":"Hike.MayFail.html#t:mapper/1","title":"Hike.MayFail.mapper/1","type":"type"},{"doc":"represent a type of Mayfail that could be in either Failure or Success state","ref":"Hike.MayFail.html#t:mayfail/0","title":"Hike.MayFail.mayfail/0","type":"opaque"},{"doc":"represent a type of Mayfail that could be in either failure or success state with a value of given type.","ref":"Hike.MayFail.html#t:mayfail/2","title":"Hike.MayFail.mayfail/2","type":"type"},{"doc":"represent a type of Mayfail in Failure state.","ref":"Hike.MayFail.html#t:mayfail_failure/0","title":"Hike.MayFail.mayfail_failure/0","type":"type"},{"doc":"represent a type of Mayfail in Failure state.","ref":"Hike.MayFail.html#t:mayfail_failure/1","title":"Hike.MayFail.mayfail_failure/1","type":"type"},{"doc":"Elevated data type of Mayfail struct that represents Success state.","ref":"Hike.MayFail.html#t:mayfail_success/0","title":"Hike.MayFail.mayfail_success/0","type":"type"},{"doc":"Elevated data type of Mayfail struct that represents Success state and have a value of type &lt;T&gt; .","ref":"Hike.MayFail.html#t:mayfail_success/1","title":"Hike.MayFail.mayfail_success/1","type":"type"},{"doc":"generic input type &lt;T&gt; .","ref":"Hike.MayFail.html#t:t/0","title":"Hike.MayFail.t/0","type":"type"},{"doc":"generic input type &lt;T_Failure&gt; represent a type of value on Failure state.","ref":"Hike.MayFail.html#t:t_failure/0","title":"Hike.MayFail.t_failure/0","type":"type"},{"doc":"generic input type &lt;T_Success&gt; represent a type of value on Success state.","ref":"Hike.MayFail.html#t:t_success/0","title":"Hike.MayFail.t_success/0","type":"type"},{"doc":"generic return type &lt;TR&gt; .","ref":"Hike.MayFail.html#t:tr/0","title":"Hike.MayFail.tr/0","type":"type"},{"doc":"The Hike.Option module provides an implementation of the Optional data type. It defines a struct Option with a single field value which can either be @none nil or any other value of type &lt;T&gt; . This implementation provides functions to work with Optional data including mapping, binding filtering, applying and many more functions to the value inside the Optional data. Example Usage iex&gt; option = % Hike.Option { value : 42 } % Hike.Option { value : 42 } iex&gt; Hike.Option . map ( option , &amp; ( &amp;1 * 2 ) ) % Hike.Option { value : 84 } iex&gt; Hike.Option . filter ( option , &amp; ( rem ( &amp;1 , 2 ) == 0 ) ) % Hike.Option { value : 42 } iex&gt; Hike.Option . apply ( option , &amp; ( &amp;1 + 10 ) ) % Hike.Option { value : 52 } # Define a User struct defmodule User do @derive { Jason.Encoder , only : [ :id , :age , :name ] } defstruct [ :id , :age , :name ] end defmodule TestHike do # Import the Hike.Option module import Hike.Option # Simulating a database fetch function @spec fetch_user ( number ) :: Hike.Option . option ( % User { } ) | Hike.Option . option ( ) # Simulating a database fetch function def fetch_user ( id ) do # Simulating a database query to fetch a user by ID # Returns an Option&lt;User&gt; with some(user) if the user is found # Returns an Option&lt;User&gt; with none() if the user is not found case id do 1 -&gt; some ( % User { id : 1 , age : 30 , name : &quot;Vineet Sharma&quot; } ) 2 -&gt; some ( % User { id : 2 , age : 20 , name : &quot;Jane Smith&quot; } ) _ -&gt; none ( ) end end # Function to update the user&#39;s name to uppercase # This function takes a user, a real data type, and returns an elevated data type Option # if `%User{}` could transform successfully return `Option` in `some` state or else in `none` state. def update_name_to_uppercase ( user ) do case user . name do nil -&gt; none ( ) name -&gt; % User { user | name : String . upcase ( user . name ) } |&gt; some end end # for above function another version could be like this where user take real data type %User{}, and return update %User{} # def update_name_to_uppercase(user) do # uppercase_name = String.upcase(user.name) # %User{user | name: uppercase_name} # end # for this case map function will be used like it&#39;s been used for `increase_age_by1` # Function to increase the user&#39;s age by one # This function takes a user, a real data type, and returns a real data type user def increase_age_by1 ( user ) do % User { user | age : user . age + 1 } end # Function to print a user struct as a JSON-represented string def print_user_as_json ( user ) do Jason . encode! ( user ) |&gt; IO . puts end # Example: Fetching a user from the database, updating the name, and matching the result def test_user ( ) do user_id = 1 # 1. Expressiveness: Using Hike&#39;s Option type to handle optional values fetch_user ( user_id ) |&gt; bind ( &amp; update_name_to_uppercase / 1 ) |&gt; map ( &amp; increase_age_by1 / 1 ) |&gt; match ( &amp; print_user_as_json / 1 , fn -&gt; IO . puts ( &quot;User not found&quot; ) end ) user_id = 3 # 2. Safer and more predictable code: Handling all possible cases explicitly fetch_user ( user_id ) |&gt; bind ( &amp; update_name_to_uppercase / 1 ) |&gt; map ( &amp; increase_age_by1 / 1 ) |&gt; match ( &amp; print_user_as_json / 1 , fn -&gt; IO . puts ( &quot;User not found&quot; ) end ) end end Output iex&gt; TestHike.test_user # User ID: 1 {&quot;id&quot;:1,&quot;age&quot;:31,&quot;name&quot;:&quot;JOHN DOE&quot;} # User ID: 3 User not found For more information on how to use this module, please see the documentation for the individual functions.","ref":"Hike.Option.html","title":"Hike.Option","type":"module"},{"doc":"Applies a given function to the value of an Option struct and returns the result as a new Option . Examples iex&gt; option = % Option { value : 42 } iex&gt; add_one = fn x -&gt; x + 1 end iex&gt; Option . apply ( option , add_one ) % Option { value : 43 } iex&gt; none_option = % Option { value : nil } iex&gt; Option . apply ( none_option , add_one ) % Option { value : nil } iex&gt; option = % Option { value : &quot;hello&quot; } iex&gt; upcase_string = fn str -&gt; String . upcase ( str ) end iex&gt; Option . apply ( option , upcase_string ) % Option { value : &quot;HELLO&quot; }","ref":"Hike.Option.html#apply/2","title":"Hike.Option.apply/2","type":"function"},{"doc":"async version on apply applies the func function to a value wrapped in a Task and returns a new task with the transformed value. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Option . some ( 10 ) end ) ...&gt; mapped_task = apply_async ( task , &amp; ( &amp;1 * 2 ) ) ...&gt; Task . await ( mapped_task ) % Hike.Option { value : 20 } iex&gt; task = Task . async ( fn -&gt; Hike.Option . none ( ) end ) iex&gt; mapped_task = apply_async ( task , &amp; ( &amp;1 ) ) iex&gt; Task . await ( mapped_task ) % Hike.Option { value : nil }","ref":"Hike.Option.html#apply_async/2","title":"Hike.Option.apply_async/2","type":"function"},{"doc":"Transforms an Option&lt;T&gt; struct with a non-nil value, using a binder function that returns another Option option in none state or Option&lt;TR&gt; option in some state. ** If the input Option has a nil value, returns a new Option struct with a nil value. if you have a function that return Option&lt;TR&gt; and you want to apply mapping then use bind function to avoid double elevation Examples iex&gt; option = % Option { value : 42 } iex&gt; add_one = fn x -&gt; Option . some ( x + 1 ) end iex&gt; Option . bind ( option , add_one ) % Option { value : 43 } iex&gt; Option . bind ( Option . some ( &quot;hello&quot; ) , fn x -&gt; Option . some ( String . upcase ( x ) ) end ) % Option { value : &quot;HELLO&quot; } iex&gt; Option . bind ( Option . none ( ) ) % Option { value : nil }","ref":"Hike.Option.html#bind/2","title":"Hike.Option.bind/2","type":"function"},{"doc":"async version on bind applies the binder function to a value wrapped in a Task and returns a new task with the transformed value. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Option . some ( 10 ) end ) ...&gt; mapped_task = bind_async ( task , fn x -&gt; Hike.Option . some ( x + 1 ) end ) ...&gt; Task . await ( mapped_task ) % Hike.Option { value : 20 } iex&gt; task = Task . async ( fn -&gt; Hike.Option . none ( ) end ) iex&gt; mapped_task = bind_async ( task , fn x -&gt; Hike.Option . some ( x + 1 ) end ) iex&gt; Task . await ( mapped_task ) % Hike.Option { value : nil }","ref":"Hike.Option.html#bind_async/2","title":"Hike.Option.bind_async/2","type":"function"},{"doc":"Applies the given function to the value of the provided option, returning a new option containing the original value if the function returns a truthy value, otherwise returning an empty option. If the provided option has no value, this function simply returns the empty option. Examples iex&gt; Option . filter ( % Option { value : 42 } , fn x -&gt; rem ( x , 2 ) == 0 end ) % Option { value : 42 } iex&gt; Option . filter ( % Option { value : 42 } , fn x -&gt; rem ( x , 2 ) == 1 end ) % Option { value : nil } iex&gt; Option . filter ( % Option { value : nil } , fn x -&gt; rem ( x , 2 ) == 0 end ) % Option { value : nil } iex&gt; list_opt = Option . some ( [ 1 , 2 , 3 ] ) iex&gt; list_filter = fn ( lst ) -&gt; Enum . count ( lst ) &gt; 5 end iex&gt; Option . filter ( list_opt , list_filter ) % Hike.Option { value : nil }","ref":"Hike.Option.html#filter/2","title":"Hike.Option.filter/2","type":"function"},{"doc":"Returns true if the Option is in None state, otherwise false . Examples iex&gt; Option . is_none? ( Option . none ( ) ) true iex&gt; Option . is_none? ( Option . some ( &quot;hello&quot; ) ) false","ref":"Hike.Option.html#is_none?/1","title":"Hike.Option.is_none?/1","type":"function"},{"doc":"Returns true if the Option is in Some state, otherwise false . Examples iex&gt; Option . is_some? ( Option . some ( &quot;hello&quot; ) ) true iex&gt; Option . is_some? ( Option . none ( ) ) false","ref":"Hike.Option.html#is_some?/1","title":"Hike.Option.is_some?/1","type":"function"},{"doc":"Applies the given mapping function to the value inside the Option struct and returns a new Option struct containing the transformed value. If the input Option struct is @none , the function returns a new Option struct in none state. Examples iex&gt; Option . map ( Option . some ( &quot;hello&quot; ) , fn x -&gt; String . upcase ( x ) end ) % Option { value : &quot;HELLO&quot; } iex&gt; Option . map ( Option . none ( ) , fn x -&gt; String . upcase ( x ) end ) % Option { value : nil }","ref":"Hike.Option.html#map/2","title":"Hike.Option.map/2","type":"function"},{"doc":"async version on map Applies the mapper function to a value wrapped in a Task and returns a new task with the mapped value. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Option . some ( 10 ) end ) ...&gt; mapped_task = map_async ( task , &amp; ( &amp;1 * 2 ) ) ...&gt; Task . await ( mapped_task ) % Hike.Option { value : 20 } iex&gt; task = Task . async ( fn -&gt; Hike.Option . none ( ) end ) iex&gt; mapped_task = map_async ( task , &amp; ( &amp;1 ) ) iex&gt; Task . await ( mapped_task ) % Hike.Option { value : nil }","ref":"Hike.Option.html#map_async/2","title":"Hike.Option.map_async/2","type":"function"},{"doc":"Pattern matches on an option and returns the result of the respective matching function. Calls some_fun with the value of the Option if the Option is in some state, or calls none_fun if the Option is in none state. Examples iex&gt; import Hike.Option # Match on `some` value with a matching function and on `none` value with a none-matching function iex&gt; match ( some ( &quot;hello&quot; ) , &amp; ( &amp;1 &lt;&gt; &quot; world&quot; ) , fn -&gt; &quot;no value&quot; end ) # =&gt; &quot;hello world&quot; # Match on `none` value with a none-matching function even if a matching function is provided iex&gt; match ( none ( ) , &amp; ( &amp;1 * 2 ) , fn -&gt; &quot;no value&quot; end ) # =&gt; &quot;no value&quot; iex&gt; Option . match ( Option . some ( &quot;hello&quot; ) , fn x -&gt; String . upcase ( x ) end , fn -&gt; &quot;nothing&quot; end ) &quot;HELLO&quot; iex&gt; Option . match ( Option . none ( ) , fn x -&gt; String . upcase ( x ) end , fn -&gt; &quot;nothing&quot; end ) &quot;nothing&quot;","ref":"Hike.Option.html#match/3","title":"Hike.Option.match/3","type":"function"},{"doc":"async version on match pattern matches on an Option value wrapped in a Task and returns the result of the matching function. Examples iex&gt; task = Task . async ( fn -&gt; Hike.Option . some ( 10 ) end ) iex&gt; mapped_task = map_async ( task , &amp; ( &amp;1 * 2 ) ) iex&gt; match_async ( mapped_task , fn x -&gt; &quot;value is : &quot; &lt;&gt; x &lt;&gt; &quot;.&quot; end , fn ( ) -&gt; &quot;No Result Found.&quot; end ) &quot;value is : 20.&quot; iex&gt; task = Task . async ( fn -&gt; Hike.Option . none ( ) end ) iex&gt; mapped_task = map_async ( task , &amp; ( &amp;1 ) ) iex&gt; match_async ( mapped_task , fn x -&gt; &quot;value is : &quot; &lt;&gt; x &lt;&gt; &quot;.&quot; end , fn ( ) -&gt; &quot;No Result Found.&quot; end ) &quot;No Result Found.&quot;","ref":"Hike.Option.html#match_async/3","title":"Hike.Option.match_async/3","type":"function"},{"doc":"Creates an Option in None state. Examples iex&gt; Option . none ( ) % Option { value : :nil }","ref":"Hike.Option.html#none/0","title":"Hike.Option.none/0","type":"function"},{"doc":"Creates an Option in Some state Examples iex&gt; Option . some ( &quot;hello&quot; ) % Option { value : &quot;hello&quot; }","ref":"Hike.Option.html#some/1","title":"Hike.Option.some/1","type":"function"},{"doc":"convert Option into respective Either . if Option is in None state Either will be return in Left state. and Option from Some state will convert to Either Right state. Example iex&gt; Hike . option ( 9 ) |&gt; Hike.Option . to_either % Hike.Either { l_value : nil , r_value : 9 , is_left? : false } iex&gt; Hike . option ( ) |&gt; Hike.Option . to_either % Hike.Either { l_value : :error , r_value : nil , is_left? : true }","ref":"Hike.Option.html#to_either/1","title":"Hike.Option.to_either/1","type":"function"},{"doc":"convert Option into respective MayFail . if Option is in None state MayFail will be return in Failure state. and Option from Some state will convert to MayFail Success state. Example iex(13)&gt; Hike . option ( ) |&gt; Hike.Option . to_mayfail % Hike.MayFail { failure : :error , success : nil , is_success? : false } iex(14)&gt; Hike . option ( 3 ) |&gt; Hike.Option . to_mayfail % Hike.MayFail { failure : nil , success : 3 , is_success? : true }","ref":"Hike.Option.html#to_mayfail/1","title":"Hike.Option.to_mayfail/1","type":"function"},{"doc":"binder() represent a binding(mapping) function which take no parameter and return an Option of type &lt;TR&gt; . Example @spec whatever ( ) :: Hike.Option . option ( atom ) def whatever ( ) , do : some ( :ok ) @spec whatever ( ) :: Hike.Option . option ( ) def whatever ( ) , do : none ( )","ref":"Hike.Option.html#t:binder/0","title":"Hike.Option.binder/0","type":"type"},{"doc":"binder(t) represent a binding(mapping) function which take a parameter of type &lt;T&gt; and return an Option of type &lt;TR&gt; . Example @spec square ( pos_number ) :: Hike.Option . option ( number ) def square ( x ) when x &gt; 0 , do : x * x |&gt; some ( ) @spec square ( number ) :: Hike.Option . option ( ) def square ( x ) do case x when x &lt; 0 do true -&gt; none ( ) false -&gt; square ( x ) end end","ref":"Hike.Option.html#t:binder/1","title":"Hike.Option.binder/1","type":"type"},{"doc":"func() represent a function which take no parameter and return value of type &lt;TR&gt; . Example @spec whatever ( ) :: atom def whatever ( ) , do : :ok","ref":"Hike.Option.html#t:func/0","title":"Hike.Option.func/0","type":"type"},{"doc":"func(t) represent a function which take a parameter of type &lt;T&gt; and return a value of type &lt;TR&gt; . Example @spec add_one ( number ) :: number def add_one ( x ) , do : x + 1","ref":"Hike.Option.html#t:func/1","title":"Hike.Option.func/1","type":"type"},{"doc":"mapper() represent a mapping function which take no parameter and return a value of type &lt;TR&gt; . Example @spec whatever ( ) :: atom def whatever ( ) , do : :ok","ref":"Hike.Option.html#t:mapper/0","title":"Hike.Option.mapper/0","type":"type"},{"doc":"mapper(t) represent a mapping function which take a parameter of type &lt;T&gt; and return a value of type &lt;TR&gt; . Example @spec square ( number ) :: number def square ( x ) , do : x * x","ref":"Hike.Option.html#t:mapper/1","title":"Hike.Option.mapper/1","type":"type"},{"doc":"Elevated data type of Option struct that represents None state. Example % Hike.Option { value : nil , is_some? : false }","ref":"Hike.Option.html#t:option/0","title":"Hike.Option.option/0","type":"type"},{"doc":"Elevated data type of Option struct that represents Some state and have a value of type &lt;T&gt; . Example % Hike.Option { value : 40 , is_some? : true }","ref":"Hike.Option.html#t:option/1","title":"Hike.Option.option/1","type":"type"},{"doc":"generic input type &lt;T&gt; .","ref":"Hike.Option.html#t:t/0","title":"Hike.Option.t/0","type":"type"},{"doc":"generic return type &lt;TR&gt; .","ref":"Hike.Option.html#t:tr/0","title":"Hike.Option.tr/0","type":"type"}]